@@ -9,7 +9,7 @@ import (
 	"path/filepath"
 	"strings"
 
-	"github.com/containers/podman/v3/pkg/rootless"
+	"github.com/containers/podman/v4/pkg/rootless"
 	systemdDbus "github.com/coreos/go-systemd/v22/dbus"
 	"github.com/cri-o/cri-o/internal/config/node"
 	"github.com/cri-o/cri-o/internal/dbusmgr"
@@ -17,7 +17,6 @@ import (
 	"github.com/godbus/dbus/v5"
 	"github.com/opencontainers/runc/libcontainer/cgroups/systemd"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/unix"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
@@ -86,10 +85,10 @@ func (m *SystemdManager) ContainerCgroupAbsolutePath(sbParent, containerID strin
 	logrus.Debugf("Expanding systemd cgroup slice %v", parent)
 	cgroup, err := systemd.ExpandSlice(parent)
 	if err != nil {
-		return "", errors.Wrapf(err, "error expanding systemd slice to get container %s stats", containerID)
+		return "", fmt.Errorf("expanding systemd slice to get container %s stats: %w", containerID, err)
 	}
 
-	return filepath.Join(cgroup, containerCgroupPath(containerID)+".scope"), nil
+	return filepath.Join(cgroup, crioPrefix+"-"+containerID+".scope"), nil
 }
 
 // MoveConmonToCgroup takes the container ID, cgroup parent, conmon's cgroup (from the config) and conmon's PID
@@ -121,7 +120,7 @@ func (m *SystemdManager) MoveConmonToCgroup(cid, cgroupParent, conmonCgroup stri
 			} else {
 				bits, err := systemd.RangeToBits(resources.CPU.Cpus)
 				if err != nil {
-					return "", errors.Wrapf(err, "cpuset conversion error")
+					return "", fmt.Errorf("cpuset conversion error: %w", err)
 				}
 				props = append(props, systemdDbus.Property{
 					Name:  "AllowedCPUs",
@@ -139,7 +138,7 @@ func (m *SystemdManager) MoveConmonToCgroup(cid, cgroupParent, conmonCgroup stri
 
 	logrus.Debugf("Running conmon under slice %s and unitName %s", cgroupParent, conmonUnitName)
 	if err := utils.RunUnderSystemdScope(m.dbusMgr, pid, cgroupParent, conmonUnitName, props...); err != nil {
-		return "", errors.Wrapf(err, "failed to add conmon to systemd sandbox cgroup")
+		return "", fmt.Errorf("failed to add conmon to systemd sandbox cgroup: %w", err)
 	}
 	// return empty string as path because cgroup cleanup is done by systemd
 	return "", nil
@@ -156,12 +155,9 @@ func (m *SystemdManager) SandboxCgroupPath(sbParent, sbID string) (cgParent, cgP
 	if !strings.HasSuffix(filepath.Base(sbParent), ".slice") {
 		return "", "", fmt.Errorf("cri-o configured with systemd cgroup manager, but did not receive slice as parent: %s", sbParent)
 	}
-	cgParent, slicePath, err := sandboxCgroupAbsolutePath(sbParent)
-	if err != nil {
-		return "", "", err
-	}
 
-	if err := verifyCgroupHasEnoughMemory(slicePath, m.memoryPath, m.memoryMaxFile); err != nil {
+	cgParent = convertCgroupFsNameToSystemd(sbParent)
+	if err := verifyCgroupHasEnoughMemory(sbParent, m.memoryPath, m.memoryMaxFile); err != nil {
 		return "", "", err
 	}
 
@@ -180,11 +176,12 @@ func (m *SystemdManager) PopulateSandboxCgroupStats(sbParent string, stats *type
 	return populateSandboxCgroupStatsFromPath(cgPath, stats)
 }
 
+// nolint: unparam // golangci-lint claims cgParent is unused, though it's being used to include documentation inline.
 func sandboxCgroupAbsolutePath(sbParent string) (cgParent, slicePath string, err error) {
 	cgParent = convertCgroupFsNameToSystemd(sbParent)
 	slicePath, err = systemd.ExpandSlice(cgParent)
 	if err != nil {
-		return "", "", errors.Wrapf(err, "expanding systemd slice path for %q", cgParent)
+		return "", "", fmt.Errorf("expanding systemd slice path for %q: %w", cgParent, err)
 	}
 	return cgParent, slicePath, nil
 }
@@ -200,37 +197,6 @@ func convertCgroupFsNameToSystemd(cgroupfsName string) string {
 }
 
 // CreateSandboxCgroup calls the helper function createSandboxCgroup for this manager.
-// Note: createSandboxCgroup will create a cgroupfs cgroup for the infra container underneath the pod slice.
-// It will not use dbus to create this cgroup, but instead call libcontainer's cgroupfs manager directly.
-// This is because a scope created here will not have a process within it (as it's usually for a dropped infra container),
-// and a slice cannot have the required `crio` prefix (while still being within the pod slice).
-// Ultimately, this cgroup is required for cAdvisor to be able to register the pod and collect network metrics for it.
-// This work will not be relevant when CRI-O is responsible for gathering pod metrics (KEP-2371), but is required until that's done.
 func (m *SystemdManager) CreateSandboxCgroup(sbParent, containerID string) error {
-	// sbParent should always be specified by kubelet, but sometimes not by critest/crictl.
-	// Skip creation in this case.
-	if sbParent == "" {
-		logrus.Infof("Not creating sandbox cgroup: sbParent is empty")
-		return nil
-	}
-	expandedParent, err := systemd.ExpandSlice(sbParent)
-	if err != nil {
-		return err
-	}
-	return createSandboxCgroup(expandedParent, containerCgroupPath(containerID))
-}
-
-// RemoveSandboxCgroup calls the helper function removeSandboxCgroup for this manager.
-func (m *SystemdManager) RemoveSandboxCgroup(sbParent, containerID string) error {
-	// sbParent should always be specified by kubelet, but sometimes not by critest/crictl.
-	// Skip creation in this case.
-	if sbParent == "" {
-		logrus.Infof("Not creating sandbox cgroup: sbParent is empty")
-		return nil
-	}
-	expandedParent, err := systemd.ExpandSlice(sbParent)
-	if err != nil {
-		return err
-	}
-	return removeSandboxCgroup(expandedParent, containerCgroupPath(containerID))
+	return createSandboxCgroup(sbParent, containerID, m)
 }
