@@ -4,17 +4,17 @@ import (
 	"bytes"
 	"context"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"time"
 
-	"github.com/containers/podman/v3/pkg/annotations"
+	"github.com/containers/podman/v4/pkg/annotations"
 	"github.com/cri-o/cri-o/internal/lib"
 	"github.com/cri-o/cri-o/internal/oci"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
 	"github.com/golang/mock/gomock"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
+	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 // The actual test suite
@@ -26,7 +26,7 @@ var _ = t.Describe("ContainerServer", func() {
 		It("should succeed with default config", func() {
 			// Given
 			// Create temp lockfile
-			tmpfile, err := ioutil.TempFile("", "lockfile")
+			tmpfile, err := os.CreateTemp("", "lockfile")
 			Expect(err).To(BeNil())
 			defer os.Remove(tmpfile.Name())
 
@@ -586,7 +586,7 @@ var _ = t.Describe("ContainerServer", func() {
 			container, err := oci.NewContainer(containerID, "", "", "",
 				make(map[string]string), make(map[string]string),
 				make(map[string]string), "", "", "",
-				&oci.Metadata{}, sandboxID, false,
+				&types.ContainerMetadata{}, sandboxID, false,
 				false, false, "", "/invalid", time.Now(), "")
 			Expect(err).To(BeNil())
 
