@@ -4,7 +4,7 @@ import (
 	"testing"
 
 	. "github.com/cri-o/cri-o/test/framework"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 )
 
